------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 214 $
--    $Date: 2008-06-30 10:45:03 +0200 (lun, 30 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
package Aadl_Config is

   -- Activate AADL Debug switch
   --
   Aadl_Debug : Boolean := False;

   -- Include system/sub-system name in Cheddar's features name
   --
   Aadl_Import_With_System_Name : Boolean := True;

   -- Transform event port to task precedencies
   --
   Aadl_Import_Event_To_Precedencies : Boolean := False;

   -- Transform event port to task precedencies
   --
   Aadl_Import_Event_Data_To_Precedencies : Boolean := False;

   -- Transform event port to buffers and messages
   -- (it automatically includes buffer/message task dependencies)
   --
   Aadl_Import_Event_To_Buffers_Messages : Boolean := True;

   -- Transform event port to buffers and messages
   -- (it automatically includes buffer/message task dependencies)
   --
   Aadl_Import_Event_Data_To_Buffers_Messages : Boolean := True;

   -- Transform task precedencies to event data port connexions
   --
   Aadl_Export_Precedencies_To_Event_Data : Boolean := False;

   -- Transform task precedencies to event port connexions
   --
   Aadl_Export_Precedencies_To_Event : Boolean := False;

   -- Transform task precedencies to a Cheddar property
   --
   Aadl_Export_Precedencies_To_Properties : Boolean := True;

   -- Process the second AADL import pass which binds software
   -- components to hardware components
   --
   Aadl_Process_Second_Import_Pass : Boolean := True;

   -- Process the third AADL import pass which build component
   -- relationships (i.e. connections)
   --
   Aadl_Process_Third_Import_Pass : Boolean := True;

   -- Display all AADL configuration variables
   --
   procedure Put;

end Aadl_Config;
