------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Events;                  use Events;
with Systems;                 use Systems;
with Call_Framework;          use Call_Framework;
with Processors;              use Processors;
with Processor_Set;           use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Ada.Strings.Unbounded;   use Ada.Strings.Unbounded;
with Scheduler;               use Scheduler;
use Scheduler.Double_Tasks_Parameters;
with Multiprocessor_Services; use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;
with Task_Set;                use Task_Set;
use Task_Set.Generic_Task_Set;

package Call_Scheduling_Framework is

   pragma Elaborate_Body;

   -- Last computed scheduling
   --
   Sched                  : Scheduling_Table_Ptr;
   Scheduling_Base_Period : Natural;

   procedure Compute_Simulation_Time_Line
     (Sys                        : in out System;
      Result                     : in out Unbounded_String;
      Period                     : in Natural;
      Schedule_With_Offsets      : in Boolean;
      Schedule_With_Precedencies : in Boolean;
      Schedule_With_Resources    : in Boolean;
      With_Task_Specific_Seed    : in Boolean       := True;
      Global_Seed_Value          : in Integer       := 0;
      Predictable_Global_Seed    : in Boolean       := True;
      Event_To_Generate          : in Time_Unit_Event_Type_Boolean_Table;
      Output                     : in Output_Format := String_Output);

   procedure Compute_Simulation_Basics
     (Sys         : in System;
      Result      : in out Unbounded_String;
      A_Processor : in Processor_Ptr;
      Output      : in Output_Format := String_Output);

   procedure Compute_Simulation_Number_Of_Preemption
     (Sys         : in System;
      Result      : in out Unbounded_String;
      A_Processor : in Processor_Ptr;
      Output      : in Output_Format := String_Output);

   procedure Compute_Simulation_Number_Of_Context_Switch
     (Sys         : in System;
      Result      : in out Unbounded_String;
      A_Processor : in Processor_Ptr;
      Output      : in Output_Format := String_Output);

   procedure Compute_Simulation_Response_Time
     (Sys          : in System;
      Result       : in out Unbounded_String;
      A_Processor  : in Processor_Ptr;
      Worst_Case   : Boolean;
      Best_Case    : Boolean;
      Average_Case : Boolean;
      Output       : in Output_Format := String_Output);

   procedure Run_User_Defined_Event_Handlers
     (Sys    : in System;
      Result : in out Unbounded_String;
      Output : in Output_Format := String_Output);

   procedure Compute_Feasibility_Basics
     (Sys         : in out System;
      Result      : in out Unbounded_String;
      A_Processor : in Processor_Ptr;
      Output      : in Output_Format := String_Output);

   procedure Compute_Feasibility_Period
     (Sys         : in System;
      Result      : in out Unbounded_String;
      A_Processor : in Processor_Ptr;
      Output      : in Output_Format := String_Output);

   procedure Compute_Feasibility_Processor_Utilization_Factor
     (Sys         : in System;
      Result      : in out Unbounded_String;
      A_Processor : in Processor_Ptr;
      Output      : in Output_Format := String_Output);

   procedure Compute_Feasibility_Response_Time
     (Sys         : in out System;
      Result      : in out Unbounded_String;
      A_Processor : in Processor_Ptr;
      Output      : in Output_Format := String_Output);

   procedure Set_Priorities
     (Sys         : in out System;
      Result      : in out Unbounded_String;
      A_Processor : in Processor_Ptr;
      A_Scheduler : in Scheduler_Type;
      Output      : in Output_Format := String_Output);

   type Partioning_Type is (
      General_Task,
      Best_Fit,
      First_Fit,
      Next_Fit,
      Small_Task);

   procedure Partionning_Tasks
     (Sys             : in out System;
      Result          : in out Unbounded_String;
      A_Parition_Rule : in Partioning_Type;
      Output          : in Output_Format := String_Output);

end Call_Scheduling_Framework;
