------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Tasks;                  use Tasks;
with Buffers;                use Buffers;
with Messages;               use Messages;
with Unchecked_Deallocation;

package Dependencies is

   type Dependency_Type is (
      Precedence_Dependency,
      Buffer_Dependency,
      Communication_Dependency);

   type Orientation_Dependency_Type is (
      From_Object_To_Task,
      From_Task_To_Object);

   type Dependency (Depend_Type : Dependency_Type) is record
      case Depend_Type is
         when Precedence_Dependency =>
            Precedency_From_Task : Generic_Task_Ptr;
            Precedency_To_Task   : Generic_Task_Ptr;

         when Buffer_Dependency =>
            Buffer_Dependent_Task    : Generic_Task_Ptr;
            Buffer_Orientation       : Orientation_Dependency_Type;
            Buffer_Dependency_Object : Buffer_Ptr;

         when Communication_Dependency =>
            Communication_Dependent_Task    : Generic_Task_Ptr;
            Communication_Orientation       : Orientation_Dependency_Type;
            Communication_Dependency_Object : Message_Ptr;

      end case;
   end record;

   type Dependency_Ptr is access all Dependency;

   procedure Initialize (My_Dep : in out Dependency_Ptr);

   procedure Put (My_Dep : in Dependency_Ptr);

   function Copy (My_Dep : in Dependency_Ptr) return Dependency_Ptr;

   procedure Free is new Unchecked_Deallocation (Dependency, Dependency_Ptr);

end Dependencies;
