------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Framework_Config;      use Framework_Config;
with Networks;              use Networks;
with Sets;

package Network_Set is

   ----------------------------------------------------------
   -- Definition of a set
   ----------------------------------------------------------

   package Generic_Network_Set is new Sets (
      Max_Element => Framework_Config.Max_Networks,
      Element => Network_Ptr,
      Free => Free,
      Copy => Copy,
      Put => Put);

   use Generic_Network_Set;

   type Networks_Set is new Generic_Network_Set.Set with private;

   subtype Network_Range is Generic_Network_Set.Element_Range;
   subtype Network_Iterator is Generic_Network_Set.Iterator;

   ----------------------------------------------------------
   -- Exceptions defined in the set package
   ----------------------------------------------------------

   -- Raised when a given network is not in a network set
   --
   Network_Not_Found : exception;

   -- Raised when 2 networks with the same name are added
   -- to a network set
   --
   Network_Already_Defined : exception;

   -- Raised when parameters provided to Add_network are wrong
   --
   Invalid_Parameter : exception;

   ----------------------------------------------------------
   -- I/O operations
   ----------------------------------------------------------
   function Export_Aadl_Implementations
     (My_Networks : in Networks_Set)
      return        Unbounded_String;

   function Export_Aadl_Declarations
     (My_Networks  : in Networks_Set;
      Number_Of_Ht : in Natural)
      return         Unbounded_String;

   function Export_Xml
     (My_Networks : in Networks_Set)
      return        Unbounded_String;

   ----------------------------------------------------------
   -- Procedure to proceed modification on a set
   ----------------------------------------------------------

   procedure Check_network
     (My_networks  : in Networks_Set;
      Name         : in Unbounded_String;
      Network_Type : in Networks_Type);

   procedure Update_Network
     (My_Networks  : in out Networks_Set;
      Name         : in Unbounded_String;
      Network_Type : in Networks_Type);

   procedure Add_Network
     (My_Networks  : in out Networks_Set;
      Name         : in Unbounded_String;
      Network_Type : in Networks_Type);

   procedure Delete_network
     (My_networks    : in out Networks_Set;
      A_network_name : in Unbounded_String);

   ------------------------------------------------------
   -- Read information from a set
   ------------------------------------------------------

   function Search_Network
     (My_Networks : in Networks_Set;
      Name        : in Unbounded_String)
      return        Network_Ptr;

private

   type Networks_Set is new Generic_Network_Set.Set with null record;

end Network_Set;
