------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Expressions;           use Expressions;
with Tables;
with Framework_Config;      use Framework_Config;
with Sections;              use Sections;
with Statements;            use Statements;

package interpreter is

   ----------------------------------------------------------
   --  Table of sections : if can be used for simple
   -- user-defined scheduler analyzer : those which have
   -- only one section of each type (ie one election section, one
   -- priority section, ...)
   ----------------------------------------------------------

   type section_table is array (Sections_Type'Range) of Generic_Statement_Ptr;

   -- section (Statements and Expressions)
   --
   procedure Put (S : in section_table);

   ---------------------------------------------------------
   -- Table to store the list of sets statements
   ---------------------------------------------------------

   package Sets_Type_Package is new Tables (
      Set_Statement_Ptr,
      Max_Statements,
      Put);
   use Sets_Type_Package;

   subtype Sets_Range is Sets_Type_Package.Table_Range;
   subtype Sets_Table_Type is Sets_Type_Package.Table;

   -- Looking for a set statement in the table
   --
   function Find_Set
     (Var  : in Sets_Table_Type;
      S    : in Unbounded_String)
      return Sets_Range;

   procedure Check_Set_Declaration
     (Var : in Sets_Table_Type;
      S   : in Unbounded_String);

   ---------------------------------------------------------------------
   --- Functions and procedure in this part of the package perform
   --  interpretation of the statements  which are part of
   --  user-defined schedulers or user-defined event analyzers
   ----------------------------------------------------------------------

   procedure Dispatch
     (Current         : in Generic_Statement_Ptr;
      Current_Section : in Sections_Type;
      processor_name  : in Unbounded_String;
      Variables_Table : in out Variables_Table_Type;
      Msg             : in out Unbounded_String);

   procedure If_Dispatch
     (Current         : in If_Statement_Ptr;
      Current_Section : in Sections_Type;
      processor_name  : in Unbounded_String;
      Variables_Table : in out Variables_Table_Type;
      Msg             : in out Unbounded_String;
      Next            : in out Generic_Statement_Ptr);

   procedure Assign_Dispatch
     (Current         : in Assign_Statement_Ptr;
      processor_name  : in Unbounded_String;
      Variables_Table : in out Variables_Table_Type;
      Msg             : in out Unbounded_String;
      Next            : in out Generic_Statement_Ptr);

   procedure Random_Initialize_Dispatch
     (Current         : in Random_Initialize_Statement_Ptr;
      processor_name  : in Unbounded_String;
      Variables_Table : in out Variables_Table_Type;
      Msg             : in out Unbounded_String;
      Next            : in out Generic_Statement_Ptr);

   procedure Put_Dispatch
     (Current         : in Put_Statement_Ptr;
      processor_name  : in Unbounded_String;
      Variables_Table : in out Variables_Table_Type;
      Msg             : in out Unbounded_String;
      Next            : in out Generic_Statement_Ptr);

   procedure While_Dispatch
     (Current         : in While_Statement_Ptr;
      Current_Section : in Sections_Type;
      processor_name  : in Unbounded_String;
      Variables_Table : in out Variables_Table_Type;
      Msg             : in out Unbounded_String;
      Next            : in out Generic_Statement_Ptr);

   procedure For_Dispatch
     (Current         : in For_Statement_Ptr;
      Current_Section : in Sections_Type;
      processor_name  : in Unbounded_String;
      Variables_Table : in out Variables_Table_Type;
      Msg             : in out Unbounded_String;
      Next            : in out Generic_Statement_Ptr);

end interpreter;
