------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
package body Random_Tools is

   function Get_Rand_Parameter
     (Min  : in Natural;
      Max  : in Natural;
      Seed : in Generator)
      return Natural
   is

      --Rand_Number : double;
      Aux : Double;
      Res : Natural;

   begin
      --Aux := Double(Drand48) * (Double(Max) - Double(Min)) + Double(Min);
      Aux := Double (Random (Seed)) * (Double (Max) - Double (Min)) +
             Double (Min);
      Res := Natural (Aux);

      if (Res < Min) or (Res > Max) then
         raise Constraint_Error;
      end if;

      return Res;
   end Get_Rand_Parameter;

   function Get_Rand_Parameter
     (Min  : in Double;
      Max  : in Double;
      Seed : in Generator)
      return Double
   is
      Res : Double := 0.0;

   begin
      Res := Double (Random (Seed)) * (Max - Min) + Min;
      --Res := Double(Drand48) * (Max - Min) + Min;

      if (Res < Min) or (Res > Max) then
         raise Constraint_Error;
      end if;

      return Res;
   end Get_Rand_Parameter;

   -- generate poisson inter arrival time (different from 0)
   --
   function Get_Exponential_Time
     (Arrival_Rate : in Double;
      Seed         : in Generator)
      return         Double
   is
      Result : Double;
   begin

      -- Tmp_Interarrival := - 1.0/Arrival_Rate * Log(Double(Drand48));
      Result := -1.0 / Arrival_Rate * Log (Double (Random (Seed)));

      return (Result);
   end Get_Exponential_Time;

end Random_Tools;
