------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Translate;      use Translate;
with Parameters;     use Parameters;
use Parameters.User_Defined_Parameters_Table_Package;
with Ada.Exceptions; use Ada.Exceptions;
with Debug;          use Debug;

package body Scheduler.User_Defined is

   procedure Set_Behavior_File_Name
     (My_Scheduler : in out user_defined_Scheduler;
      To_Set       : in Unbounded_String)
   is
   begin
      My_Scheduler.Behavior_File_Name := To_Set;
   end Set_Behavior_File_Name;

   function Get_Behavior_File_Name
     (My_Scheduler : in user_defined_Scheduler)
      return         Unbounded_String
   is
   begin
      return My_Scheduler.Behavior_File_Name;
   end Get_Behavior_File_Name;

   procedure Set_String_Behavior
     (My_Scheduler : in out user_defined_Scheduler;
      To_Set       : in Unbounded_String)
   is
   begin
      My_Scheduler.String_Behavior     := new Unbounded_String;
      My_Scheduler.String_Behavior.all := To_Set;
   end Set_String_Behavior;

   function Get_String_Behavior
     (My_Scheduler : in user_defined_Scheduler)
      return         Unbounded_String
   is
   begin
      return My_Scheduler.String_Behavior.all;
   end Get_String_Behavior;

   function Get_String_Behavior
     (My_Scheduler : in user_defined_Scheduler)
      return         Unbounded_String_Ptr
   is
   begin
      return My_Scheduler.String_Behavior;
   end Get_String_Behavior;

   procedure Compute_Blocking_Time
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in Tasks_Set;
      My_Resources   : in Resources_Set;
      Processor_Name : in Unbounded_String;
      Msg            : in out Unbounded_String;
      Blocking_Time  : out Blocking_Time_Table)
   is
   begin
      raise Invalid_Scheduler;
   end Compute_Blocking_Time;

   procedure Bound_On_Processor_Utilization
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Double;
      Msg            : in out Unbounded_String)
   is

   begin
      raise Invalid_Scheduler;
   end Bound_On_Processor_Utilization;

   procedure Compute_Response_Time
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in out Tasks_Set;
      Processor_Name : in Unbounded_String;
      Msg            : in out Unbounded_String;
      Response_Time  : out Response_Time_Table)
   is

   begin
      raise Invalid_Scheduler;
   end Compute_Response_Time;

   procedure Check_Before_Scheduling
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String)
   is

   begin
      null;
   end Check_Before_Scheduling;

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String)
   is

   begin
      Result := Result &
                Lb_Compute_Scheduling_Error_19 (Current_Language) &
                Unbounded_Lf;
   end Utilization_Factor_Feasibility_Test;

end Scheduler.User_Defined;
