------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Unbounded_Strings; use Unbounded_Strings;

package Scheduler.user_defined is

   type user_defined_Scheduler is abstract new Generic_Scheduler with private;
   type user_defined_Scheduler_Ptr is access all user_defined_Scheduler'Class;

   procedure Bound_On_Processor_Utilization
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Double;
      Msg            : in out Unbounded_String);

   procedure Compute_Blocking_Time
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in Tasks_Set;
      My_Resources   : in Resources_Set;
      Processor_Name : in Unbounded_String;
      Msg            : in out Unbounded_String;
      Blocking_Time  : out Blocking_Time_Table);

   procedure Compute_Response_Time
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in out Tasks_Set;
      Processor_Name : in Unbounded_String;
      Msg            : in out Unbounded_String;
      Response_Time  : out Response_Time_Table);

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String);

   procedure Check_Before_Scheduling
     (My_Scheduler   : in user_defined_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String);

   procedure Set_Behavior_File_Name
     (My_Scheduler : in out user_defined_Scheduler;
      To_Set       : in Unbounded_String);

   function Get_Behavior_File_Name
     (My_Scheduler : in user_defined_Scheduler)
      return         Unbounded_String;

   procedure Set_String_Behavior
     (My_Scheduler : in out user_defined_Scheduler;
      To_Set       : in Unbounded_String);

   function Get_String_Behavior
     (My_Scheduler : in user_defined_Scheduler)
      return         Unbounded_String;

   function Get_String_Behavior
     (My_Scheduler : in user_defined_Scheduler)
      return         Unbounded_String_Ptr;

private

   type user_defined_Scheduler is abstract new Generic_Scheduler with record

   -- Scheduler behavior in string format
   --
      String_Behavior : Unbounded_String_Ptr;

      -- Scheduler behavior in string format
      --
      Behavior_File_Name : Unbounded_String;

   end record;

end Scheduler.user_defined;
