------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
package body Tasks.Extended is

   procedure Put (My_Task_Occurence : in Task_Occurence) is

   begin

      Put ("Arrival_Date : " & My_Task_Occurence.Arrival_Date'Img & " ; ");
      Put ("Absolute_Deadline : " & My_Task_Occurence.Absolute_Deadline'Img);
      Put ("Residual_capacity : " & My_Task_Occurence.Residual_Capacity'Img);
      Put ("Response_Time : " & My_Task_Occurence.Response_Time'Img);
   end Put;

   procedure Initialize_Cyclic
     (My_Task_Occurence : in out Task_Occurence;
      A_Task            : in Generic_Task_Ptr;
      Arrival_Date      : in Natural := 0)
   is
   begin
      My_Task_Occurence.Parent_Task       := A_Task;
      My_Task_Occurence.Residual_Capacity := A_Task.Capacity;
      My_Task_Occurence.Arrival_Date      := Arrival_Date;
      My_Task_Occurence.Absolute_Deadline := My_Task_Occurence.Arrival_Date +
                                             A_Task.Deadline;

   end Initialize_Cyclic;

   procedure Initialize_Acyclic
     (My_Task_Occurence : in out Task_Occurence;
      A_Task            : in Aperiodic_Task_Ptr)
   is
   begin
      My_Task_Occurence.Parent_Task       := Generic_Task_Ptr (A_Task);
      My_Task_Occurence.Residual_Capacity := A_Task.Capacity;
      My_Task_Occurence.Arrival_Date      := A_Task.Start_Time;
      My_Task_Occurence.Absolute_Deadline := My_Task_Occurence.Arrival_Date +
                                             A_Task.Deadline;
   end Initialize_Acyclic;

end Tasks.Extended;
