------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Gdk.Bitmap;       use Gdk.Bitmap;
with Gdk.Pixmap;       use Gdk.Pixmap;
with Gtk.Table;        use Gtk.Table;
with Gtk.Label;        use Gtk.Label;
with Gtk.Separator;    use Gtk.Separator;
with Gtk.Button;       use Gtk.Button;
with Gtk.Check_Button; use Gtk.Check_Button;
with Gtk.Hbutton_Box;  use Gtk.Hbutton_Box;
with Gtk.Window;       use Gtk.Window;
with Gtk.Box;          use Gtk.Box;
with Gtk.Frame;        use Gtk.Frame;
with Gtk.Notebook;     use Gtk.Notebook;
with Gtk.Pixmap;       use Gtk.Pixmap;

package Aadl_Options_Pkg is

   type Aadl_Options_Record is new Gtk_Window_Record with private;
   type Aadl_Options_Access is access all Aadl_Options_Record'Class;

   procedure Gtk_New (Aadl_Options : out Aadl_Options_Access);
   procedure Initialize (Aadl_Options : access Aadl_Options_Record'Class);

   Aadl_Options : Aadl_Options_Access;

private

   type Aadl_Options_Record is new Gtk_Window_Record with record
      Options_Vbox : Gtk_Vbox;

      Value_Table : Gtk_Table;
      Label       : Gtk_Label;

      -- Activate AADL Debug switch
      --
      Aadl_Debug : Gtk_Check_Button;

      -- Include system/sub-system name in Cheddar's features name
      --
      Aadl_Import_With_System_Name : Gtk_Check_Button;

      -- Transform event port ad event data port to task precedencies
      --
      Aadl_Import_Event_To_Precedencies      : Gtk_Check_Button;
      Aadl_Import_Event_Data_To_Precedencies : Gtk_Check_Button;

      -- Transform event port and event data port to buffers and messages
      -- (it automatically includes buffer/message task dependencies)
      --
      Aadl_Import_Event_To_Buffers_Messages      : Gtk_Check_Button;
      Aadl_Import_Event_Data_To_Buffers_Messages : Gtk_Check_Button;

      -- Transform task precedencies to a Cheddar property
      --
      Aadl_Export_Precedencies_To_Properties : Gtk_Check_Button;

      -- Transform task precedencies to event port connections
      --
      Aadl_Export_Precedencies_To_Event : Gtk_Check_Button;

      -- Transform task precedencies to event data port connections
      --
      Aadl_Export_Precedencies_To_Event_Data : Gtk_Check_Button;

      -- Process the second AADL import pass which binds software components
      --to hardware components
      --
      Aadl_Process_Second_Import_Pass : Gtk_Check_Button;

      -- Process the third AADL import pass which build component
      --relationships (i.e. connections)
      --
      Aadl_Process_Third_Import_Pass : Gtk_Check_Button;

      Button_Hbox   : Gtk_Hbutton_Box;
      Ok_Button     : Gtk_Button;
      Cancel_Button : Gtk_Button;
      Hseparator    : Gtk_Hseparator;

      Main_Frame : Gtk_Frame;
      Child_Page : Gtk_Frame;
      Label_Box  : Gtk_Box;
      Menu_Box   : Gtk_Box;
      Pixmap     : Gtk_Pixmap;
      Lab        : Gtk_Label;

      Book_Opened      : Gdk_Pixmap;
      Book_Opened_Mask : Gdk_Bitmap;
      Book_Closed      : Gdk_Pixmap;
      Book_Closed_Mask : Gdk_Bitmap;
      Notebook         : Gtk_Notebook;
   end record;

end Aadl_Options_Pkg;
