------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Message_Set;             use Message_Set;
use Message_Set.Generic_Message_Set;
with Graph_Item_Pkg;          use Graph_Item_Pkg;
with Graph_Window_Pkg;        use Graph_Window_Pkg;
with My_Canvas;               use My_Canvas;
with Update_Messages_Set_Pkg; use Update_Messages_Set_Pkg;
with Update_Buffers_Set_Pkg;  use Update_Buffers_Set_Pkg;
with Update_Tasks_Set_Pkg;    use Update_Tasks_Set_Pkg;
with Gtk.GEntry;              use Gtk.GEntry;
with Translate;               use Translate;
with Project;                 use Project;
with Framework_Config;        use Framework_Config;

package body Graph_Add_Pkg.Callbacks is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is
      Name : Unbounded_String;
      Item : Item_Object_Access;
   begin

      Is_Saved := False;

      Name :=
         To_Unbounded_String
           (Gtk.GEntry.Get_Text (Get_Entry (Add_New_Item.Combo)));
      if (Add_New_Item.Item_Type = "message") then
         if (Name = To_String (Lb_New (Current_Language))) then
            Gtk_New
              (Update_Messages_Set,
               To_Unbounded_String ("graph_window"));
            Show_Advanced (Update_Messages_Set, False);

         else
            Item :=
              Item_Object_Access (Search_Item_By_Name
                                     (Graph_Window.Buffer,
                                      To_Unbounded_String ("message"),
                                      Name));
            -- one item has been added but it already exists
            Last_Item := Last_Item - 1;

            Set_Visibility (Item, True);
            Refresh_Canvas (Graph_Window.Buffer);
         end if;
      end if;

      if (Add_New_Item.Item_Type = "buffer") then
         if (Name = To_String (Lb_New (Current_Language))) then
            Gtk_New
              (Update_Buffers_Set,
               Lb_Update_Buffer (Current_Language),
               To_Unbounded_String ("graph_window"));
            Show_Advanced (Update_Buffers_Set, False);
         else
            Item :=
              Item_Object_Access (Search_Item_By_Name
                                     (Graph_Window.Buffer,
                                      To_Unbounded_String ("buffer"),
                                      Name));
            -- one item has been added but it already exists
            Last_Item := Last_Item - 1;

            Set_Visibility (Item, True);
            Refresh_Canvas (Graph_Window.Buffer);
         end if;
      end if;

      if (Add_New_Item.Item_Type = "task") then
         if (Name = To_String (Lb_New (Current_Language))) then
            Gtk_New (Update_Tasks_Set, Lb_Update_Task (Current_Language));
            Show_Advanced (Update_Tasks_Set, False);
         else
            Item :=
              Item_Object_Access (Search_Item_By_Name
                                     (Graph_Window.Buffer,
                                      To_Unbounded_String ("task"),
                                      Name));
            -- one item has been added but it already exist
            Last_Item := Last_Item - 1;

            Set_Visibility (Item, True);
            Refresh_Canvas (Graph_Window.Buffer);
         end if;
      end if;

      Destroy (Add_New_Item);

   end On_Ok_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      -- one item has been added but it already exists
      Last_Item := Last_Item - 1;

      Destroy (Add_New_Item);
   end On_Cancel_Pressed;

   function On_Graph_Add_Delete_Event
     (Object : access Gtk_Widget_Record'Class;
      Params : Gtk.Arguments.Gtk_Args)
      return   Boolean
   is
   begin
      -- one item has been added but it already exists
      Last_Item := Last_Item - 1;

      Destroy (Add_New_Item);

      return True;
   end On_Graph_Add_Delete_Event;

end Graph_Add_Pkg.Callbacks;
