------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Gdk;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with My_Canvas;             use My_Canvas;
with Gdk.Color;             use Gdk.Color;
with Glib;                  use Glib;
with Gdk.Pixmap;            use Gdk.Pixmap;
with Gdk.Font;              use Gdk.Font;
with Framework_Config;      use Framework_Config;

package Graph_Item_Pkg is

   Items_List :
     array (1 .. Max_Messages + Max_Buffers + Max_Tasks + 1) of Canvas_Item;
   Last_Item  : Positive          := 1;
   Last_Link  : Positive          := 1;
   Font       : Gdk.Font.Gdk_Font := Gdk.Font.Null_Font;

   type Item_Object_Record is new Canvas_Item_Record with record
      Color    : Gdk.Color.Gdk_Color;
      W, H     : Gint;
      Selected : Boolean := False;
   end record;
   type Item_Object_Access is access all Item_Object_Record'Class;

   procedure Item_Object_New
     (Item  : out Item_Object_Access;
      Fig   : in Unbounded_String;
      Start : in Boolean);

   procedure Intialize_Task
     (Item  : access Item_Object_Record;
      Start : in Boolean);
   procedure Intialize_Message
     (Item  : access Item_Object_Record;
      Start : in Boolean);
   procedure Intialize_Buffer
     (Item  : access Item_Object_Record;
      Start : in Boolean);

   procedure Draw
     (Item         : access Item_Object_Record;
      Canvas       : access Interactive_Canvas_Record'Class;
      Dest         : Gdk.Pixmap.Gdk_Pixmap;
      Xdest, Ydest : Glib.Gint);

   procedure Draw_Task
     (Item         : access Item_Object_Record;
      Canvas       : access Interactive_Canvas_Record'Class;
      Dest         : Gdk.Pixmap.Gdk_Pixmap;
      Xdest, Ydest : Glib.Gint);
   procedure Draw_Message
     (Item         : access Item_Object_Record;
      Canvas       : access Interactive_Canvas_Record'Class;
      Dest         : Gdk.Pixmap.Gdk_Pixmap;
      Xdest, Ydest : Glib.Gint);
   procedure Draw_Buffer
     (Item         : access Item_Object_Record;
      Canvas       : access Interactive_Canvas_Record'Class;
      Dest         : Gdk.Pixmap.Gdk_Pixmap;
      Xdest, Ydest : Glib.Gint);

   procedure Add_Canvas_Link
     (Canvas       : access Interactive_Canvas_Record'Class;
      Item1, Item2 : access Canvas_Item_Record'Class;
      Text         : String := "");

   procedure Remove_Link_Between
     (Canvas       : access Interactive_Canvas_Record'Class;
      Item1, Item2 : access Item_Object_Record'Class);

   procedure Remove_Item
     (Canvas : access Interactive_Canvas_Record'Class;
      Item   : access Item_Object_Record'Class);

   procedure Remove_Item2
     (Canvas : access Interactive_Canvas_Record'Class;
      Item   : access Item_Object_Record'Class);
   -- this procedure remove an item from the item array and
   -- remove the corresponding graphical representation (if
   -- it exists)

   procedure Remove_Item_Link
     (Canvas : access Interactive_Canvas_Record'Class;
      Item   : access Item_Object_Record'Class);

end Graph_Item_Pkg;
