------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Gdk.GC;              use Gdk.GC;
with Gtk.Box;             use Gtk.Box;
with Gtk.Scrolled_Window; use Gtk.Scrolled_Window;
with Gtk.Window;          use Gtk.Window;
with Gtk.Viewport;        use Gtk.Viewport;
with My_Canvas;           use My_Canvas;
with Glib;                use Glib;

package Graph_Window_Pkg is

   -- Graph type specification
   type Graph_Type is (G1, G2, G3);

   type Graph_Window_Record is new Gtk_Window_Record with record
      Scrolled : Gtk_Scrolled_Window;
      Buffer   : Interactive_Canvas;
      Green_Gc : Gdk_GC;
      Viewport : Gtk_Viewport;
      V_Box    : Gtk_Vbox;

      Cur_Graph_Type : Graph_Type;

      G1_Borders_Space       : Guint;
      G1_Space_Between_Items : Guint;
      G2_Borders_Space       : Guint;
      G2_Force_Circle        : Boolean;
      G3_Borders_Space       : Guint;
      G3_Space_Between_Items : Guint;
   end record;

   type Graph_Window_Access is access all Graph_Window_Record;

   procedure Rebuild_Canvas (Graph_Window : Graph_Window_Access);
   procedure Import_Task (Graph_Window : Graph_Window_Access);
   procedure Import_Message (Graph_Window : Graph_Window_Access);
   procedure Import_Buffer (Graph_Window : Graph_Window_Access);
   procedure Import_Dependencies (Graph_Window : Graph_Window_Access);

   procedure Gtk_New (Graph_Window : in out Graph_Window_Access);

   procedure Initialize (Graph_Window : Graph_Window_Access);

   Selection_Mode    : Boolean   := True;
   Max_Size          : constant := 400;
   Is_Hidden         : Boolean   := False;
   Graph_Window      : Graph_Window_Access;
   Canvas_Is_Created : Boolean   := False;

   -- Default window size
   Default_Window_Width  : constant Gint := 400;
   Default_Window_Height : constant Gint := 300;

   -- Canvas item specification
   Item_Width  : constant Gint := 35;
   Item_Height : constant Gint := 35;

end Graph_Window_Pkg;
