------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Translate;         use Translate;
with Gtk;               use Gtk;
with Gtk.Widget;        use Gtk.Widget;
with Gtk.Enums;         use Gtk.Enums;
with Callbacks_Cheddar; use Callbacks_Cheddar;
with Framework_Config;  use Framework_Config;

package body read_sc_file_Pkg is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is

   begin
      read_sc_file.procedure_to_call
        (To_Unbounded_String (Get_Filename (read_sc_file)));
      Destroy (read_sc_file);
   end On_Ok_Pressed;

   procedure On_cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Get_Toplevel (Gtk_Widget (Object)));
   end On_cancel_Pressed;

   procedure Gtk_New
     (read_sc_file : out read_sc_file_Access;
      proc         : in return_file_name)
   is

   begin
      read_sc_file := new read_sc_file_Record;
      read_sc_file_Pkg.Initialize (read_sc_file, proc);
   end Gtk_New;

   procedure Initialize
     (read_sc_file : access read_sc_file_Record'Class;
      proc         : in return_file_name)
   is
   begin
      read_sc_file.procedure_to_call := proc;
      Gtk.File_Selection.Initialize
        (read_sc_file,
         To_String
            (Lb_Read (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Show_File_Op_Buttons (read_sc_file, True);
      Set_Border_Width (read_sc_file, 10);
      Set_Title
        (read_sc_file,
         To_String
            (Lb_Read (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Policy (read_sc_file, False, True, False);
      Set_Position (read_sc_file, Win_Pos_Center_Always);
      Set_Modal (read_sc_file, True);

      read_sc_file.Ok_Button := Get_Ok_Button (read_sc_file);
      Set_Flags (read_sc_file.Ok_Button, Can_Default);
      Button_Callback.Connect
        (read_sc_file.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      read_sc_file.Cancel_Button := Get_Cancel_Button (read_sc_file);
      Set_Flags (read_sc_file.Cancel_Button, Can_Default);
      Button_Callback.Connect
        (read_sc_file.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_cancel_Pressed'Access));

   end Initialize;

end read_sc_file_Pkg;
