------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with GNAT.Current_Exception; use GNAT.Current_Exception;
with Unbounded_Strings;      use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                use Project;
with Processor_Set;          use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Task_Set;               use Task_Set;
use Task_Set.Generic_Task_Set;
with Buffer_Set;             use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resource_Set;           use Resource_Set;
use Resource_Set.Generic_Resource_Set;

package body Update_Event_Analyzers_Set_Pkg.Callbacks.Integrity_Checks is

   procedure Check_Integrity_Constraints
     (Update_Event_Analyzers_Set : in out Update_Event_Analyzers_Set_Access;
      Error_Message              : in out Unbounded_String;
      Ok                         : in out Boolean)
   is

   begin

      Ok := True;

      Event_Analyzer_Name :=
         To_Unbounded_String
           (Get_Text (Update_Event_Analyzers_Set.Entry_File_Name));

      Check_Event_Analyzer
        (Sys.Event_Analyzers,
         Integrity_Checks.Event_Analyzer_Name);

   exception
      when others =>
         Ok            := False;
         Error_Message := To_Unbounded_String (Exception_Message);

   end Check_Integrity_Constraints;

end Update_Event_Analyzers_Set_Pkg.Callbacks.Integrity_Checks;
