------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 97 $
--    $Date: 2007-07-20 15:17:15 +0200 (Fri, 20 Jul 2007) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with GNAT.Current_Exception; use GNAT.Current_Exception;
with Glib;                   use Glib;
with Translate;              use Translate;
with Framework_Config;       use Framework_Config;
with Unbounded_Strings;      use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                use Project;
with Gtk.Radio_Menu_Item;    use Gtk.Radio_Menu_Item;
with Processors;             use Processors;
with Task_Set;               use Task_Set;
use Task_Set.Generic_Task_Set;
with Buffer_Set;             use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resource_Set;           use Resource_Set;
use Resource_Set.Generic_Resource_Set;

package body Update_networks_Set_Pkg.Callbacks.Integrity_Checks is

   procedure Check_Integrity_Constraints
     (update_networks_Set : in out Update_networks_Set_Access;
      Error_Message       : in out Unbounded_String;
      Ok                  : in out Boolean)
   is

      protocol : Gint;

   begin

      Ok := True;

      network_Name :=
         To_Unbounded_String (Get_Text (Update_networks_Set.Name));

      protocol :=
         Gint'Val
           (2 -
            Gint (Selected_Button (Update_networks_Set.Clist_protocol_Menu)));

      if (protocol = 0) then
         network_protocol := Bounded_Delay;
      end if;
      if (protocol = 1) then
         network_protocol := jitter_delay;
      end if;
      if (protocol = 2) then
         network_protocol := Parametric_Delay;
      end if;

      Check_network
        (Sys.Networks,
         Integrity_Checks.network_Name,
         Integrity_Checks.network_protocol);

   exception
      when others =>
         Ok            := False;
         Error_Message := To_Unbounded_String (Exception_Message);

   end Check_Integrity_Constraints;

end update_networks_Set_Pkg.Callbacks.Integrity_Checks;
