------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with GNAT.Current_Exception;                             use 
  GNAT.Current_Exception;
with Glib;                                               use Glib;
with Mmi_Tools;                                          use Mmi_Tools;
with Translate;                                          use Translate;
with Framework_Config;                                   use Framework_Config;
with Unbounded_Strings;                                  use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                                            use Project;
with Networks;                                           use Networks;
with Gtk.Radio_Menu_Item;                                use 
  Gtk.Radio_Menu_Item;
with Graph_Window_Pkg;                                   use Graph_Window_Pkg;
with Update_networks_Set_Pkg.Callbacks.Integrity_Checks;
with My_Canvas;                                          use My_Canvas;

package body Update_networks_Set_Pkg.Callbacks is

   procedure Show_networks
     (Update_networks_Set : access Update_networks_Set_Record'Class)
   is

      a_network        : Network_Ptr;
      network_protocol : Unbounded_String;
      My_Iterator      : Network_Iterator;
      Line             : Gint;
      network_Share    : Chars_Ptr_Array (1 .. 2);

   begin
      Clear (Update_networks_Set.networks_List);

      Reset_Iterator (Sys.Networks, My_Iterator);
      if not Is_Empty (Sys.Networks) then
         loop

            Current_Element (Sys.Networks, a_network, My_Iterator);

            network_protocol :=
               To_Unbounded_String
                 (Networks_Type'Image (a_network.Network_Type));

            -- Create the line
            --
            network_Share := To_String (a_network.Name) +
                             To_String (network_protocol);

            -- Display the Line
            --
            Line := Append (Update_networks_Set.networks_List, network_Share);

            exit when Is_Last_Element (Sys.Networks, My_Iterator);

            Next_Element (Sys.Networks, My_Iterator);

         end loop;
      end if;
   end Show_networks;

   procedure On_Click_List (List : access Gtk_Clist_Record'Class) is

      Name             : Unbounded_String;
      Row              : Gint;
      Index            : Strings_Range := Strings_Range'First;
      a_network        : Network_Ptr;
      network_protocol : Networks_Type;

      Items       : Strings_Table;
      protocol_Id : Gint := 0;

      Location_Items : Strings_Table;

   begin

      Row  := Get_Focus_Row (Update_networks_Set.networks_List);
      Name :=
         To_Unbounded_String
           (Get_Text (Update_networks_Set.networks_List, Row, 0));

      Update_networks_Set.Current_network := Name;
      a_network                           :=
         Search_Network (Sys.Networks, Name);

      network_protocol := a_network.Network_Type;

      Gtk.GEntry.Set_Text (Update_networks_Set.Name, To_String (Name));

      -- network protocol
      --
      if (network_protocol = Bounded_Delay) then
         protocol_Id := 0;
      end if;
      if (network_protocol = jitter_delay) then
         protocol_Id := 1;
      end if;
      if (network_protocol = Parametric_Delay) then
         protocol_Id := 2;
      end if;

      Update_networks_Set.Clist_protocol_Menu := Widget_SList.Null_List;
      Gtk.Option_Menu.Gtk_New (Update_networks_Set.protocol_Menu);
      Add (Items, To_Unbounded_String ("Bounded delay"));
      Add (Items, To_Unbounded_String ("Jitter delay"));
      Add (Items, To_Unbounded_String ("Parametric delay"));
      Build_Option_Menu
        (Update_networks_Set.protocol_Menu,
         Update_networks_Set.Clist_protocol_Menu,
         Items,
         protocol_Id,
         null);
      Show_All (Update_networks_Set.protocol_Menu);
      Attach_Defaults
        (Update_networks_Set.network_Table,
         Update_networks_Set.protocol_Menu,
         1,
         2,
         1,
         2);

      Show_All (Update_networks_Set);

   end On_Click_List;

   procedure On_Close_Pressed (Object : access Gtk_Button_Record'Class) is
   begin

      Is_New   := False;
      Is_Saved := False;

      Destroy (Update_networks_Set);
   end On_Close_Pressed;

   procedure On_Delete_Pressed (Object : access Gtk_Button_Record'Class) is

      My_Iterator : Network_Set.Generic_Network_Set.Iterator;

      a_network : Network_Ptr;

   begin

      if Update_networks_Set.Current_network = "" then
         Show_Message_Box (Lb_Select_network_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Networks) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Reset_Iterator (Sys.Networks, My_Iterator);
      if not Is_Empty (Sys.Networks) then
         loop

            Current_Element (Sys.Networks, a_network, My_Iterator);
            if a_network.Name = Update_networks_Set.Current_network then

               Delete_network (Sys.Networks, a_network.Name);

               if (Graph_Window /= null) then
                  Refresh_Canvas (Graph_Window.Buffer);
               end if;

               -- Clear  entries
               --
               Update_networks_Set.Current_network := To_Unbounded_String
                                                        ("");
               Gtk.GEntry.Set_Text (Update_networks_Set.Name, "");
               exit;
            end if;

            exit when Is_Last_Element (Sys.Networks, My_Iterator);
            Next_Element (Sys.Networks, My_Iterator);

         end loop;
      end if;

      Show_networks (Update_networks_Set);

   end On_Delete_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Duplicate (Update_networks_Set.Undo, Sys);
      Rebuild_Canvas (Graph_Window);
      Destroy (Update_networks_Set);
   end On_Cancel_Pressed;

   procedure On_Advanced_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      if Visible_Is_Set (Update_networks_Set.Vbox_List) then
         Show_Advanced (Update_networks_Set, False);
      else
         Show_Advanced (Update_networks_Set, True);
      end if;
   end On_Advanced_Pressed;

   procedure On_Add_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      Integrity_Checks.Check_Integrity_Constraints
        (Update_networks_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      -- Add the new item in the set
      --
      Add_Network
        (Sys.Networks,
         Integrity_Checks.network_Name,
         Integrity_Checks.network_protocol);

      Update_networks_Set.Current_network := Integrity_Checks.network_Name;

      Show_networks (Update_networks_Set);

   exception
      when Network_Set.Invalid_Parameter =>
         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Add_Pressed;

   procedure On_Modify_Pressed (Object : access Gtk_Button_Record'Class) is

      a_network   : Network_Ptr;
      My_Iterator : Network_Set.Generic_Network_Set.Iterator;

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      if Update_networks_Set.Current_network = "" then
         Show_Message_Box (Lb_Select_network_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Networks) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Integrity_Checks.Check_Integrity_Constraints
        (Update_networks_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      --
      -- Update set
      --

      Reset_Iterator (Sys.Networks, My_Iterator);
      loop

         Current_Element (Sys.Networks, a_network, My_Iterator);
         if a_network.Name = Update_networks_Set.Current_network then

            Delete (Sys.Networks, a_network);

            Add_Network
              (Sys.Networks,
               Integrity_Checks.network_Name,
               Integrity_Checks.network_protocol);

            Update_networks_Set.Current_network :=
              Integrity_Checks.network_Name;
            exit;
         end if;

         exit when Is_Last_Element (Sys.Networks, My_Iterator);
         Next_Element (Sys.Networks, My_Iterator);

      end loop;

      Show_networks (Update_networks_Set);

   exception
      when Network_Set.Invalid_Parameter =>
         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Modify_Pressed;

end Update_networks_Set_Pkg.Callbacks;
