------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Glib;                              use Glib;
with Mmi_Tools;                         use Mmi_Tools;
with Callbacks_Cheddar;                 use Callbacks_Cheddar;
with Update_networks_Set_Pkg.Callbacks; use Update_networks_Set_Pkg.Callbacks;
with Unbounded_Strings;                 use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                           use Project;
with Translate;                         use Translate;
with Framework_Config;                  use Framework_Config;

package body Update_networks_Set_Pkg is

   procedure Gtk_New
     (Update_networks_Set : out Update_networks_Set_Access;
      Title               : in Unbounded_String)
   is
   begin
      Update_networks_Set := new Update_networks_Set_Record;
      Update_networks_Set_Pkg.Initialize (Update_networks_Set, Title);
   end Gtk_New;

   procedure Show_Advanced
     (Update_networks_Set : in Update_networks_Set_Access;
      Advanced            : in Boolean)
   is

   begin

      if not Advanced then
         Hide (Update_networks_Set.Vbox_List);
         Hide (Update_networks_Set.Delete_Button);
         Hide (Update_networks_Set.Modify_Button);
         Show (Update_networks_Set.Close_Button);
         Show (Update_networks_Set.Cancel_Button);
         Show (Update_networks_Set.Advanced_Button);

         Show (Update_networks_Set.Vbox_Main);
         Show (Update_networks_Set.Hbox_network);
         Show_All (Update_networks_Set.Vbox_Update);
         Show (Update_networks_Set.Hbutton_Box_Modify);
         Show (Update_networks_Set.Frame_network);

      else
         Show_All (Update_networks_Set.Vbox_Main);
      end if;
      Show (Update_networks_Set);

   end Show_Advanced;

   procedure Initialize
     (Update_networks_Set : access Update_networks_Set_Record'Class;
      Title               : in Unbounded_String)
   is

      Items : Strings_Table;

   begin

      Initialize (Update_networks_Set.Undo);
      Duplicate (Sys, Update_networks_Set.Undo);
      Update_networks_Set.Current_network := To_Unbounded_String ("");

      -- List of network protocol
      --
      Initialize (Items);
      Add (Items, To_Unbounded_String ("Bounded delay"));
      Add (Items, To_Unbounded_String ("Jitter delay"));
      Add (Items, To_Unbounded_String ("Parametric delay"));

      Gtk.Window.Initialize (Update_networks_Set, Window_Toplevel);
      Set_Policy (Update_networks_Set, True, True, True);
      Set_Title (Update_networks_Set, To_String (Title));
      Set_Position (Update_networks_Set, Win_Pos_Center_Always);
      Set_Modal (Update_networks_Set, True);
      Grab_Focus (Update_networks_Set);

      Gtk.Box.Gtk_New_Vbox (Update_networks_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_networks_Set.Vbox_Main, 10);
      Add (Update_networks_Set, Update_networks_Set.Vbox_Main);

      Gtk.Box.Gtk_New_Hbox (Update_networks_Set.Hbox_network, False, 10);
      Add (Update_networks_Set.Vbox_Main, Update_networks_Set.Hbox_network);

      Gtk.Box.Gtk_New_Vbox (Update_networks_Set.Vbox_Update, False, 10);
      Add (Update_networks_Set.Hbox_network, Update_networks_Set.Vbox_Update);

      Gtk.Box.Gtk_New_Vbox (Update_networks_Set.Vbox_List, False, 10);
      Add (Update_networks_Set.Hbox_network, Update_networks_Set.Vbox_List);

      Gtk.Frame.Gtk_New (Update_networks_Set.Frame_network, "");
      Add
        (Update_networks_Set.Vbox_Update,
         Update_networks_Set.Frame_network);

      Gtk_New (Update_networks_Set.Hbutton_Box_Modify);
      Set_Spacing (Update_networks_Set.Hbutton_Box_Modify, 10);
      Set_Layout
        (Update_networks_Set.Hbutton_Box_Modify,
         Buttonbox_Default_Style);
      Set_Child_Size (Update_networks_Set.Hbutton_Box_Modify, 20, 20);
      Set_Child_Ipadding (Update_networks_Set.Hbutton_Box_Modify, 7, 0);
      Pack_Start
        (Update_networks_Set.Vbox_Update,
         Update_networks_Set.Hbutton_Box_Modify,
         False,
         True,
         5);

      Gtk.Table.Gtk_New (Update_networks_Set.network_Table, 2, 2, False);
      Set_Border_Width (Update_networks_Set.network_Table, 5);
      Add
        (Update_networks_Set.Frame_network,
         Update_networks_Set.network_Table);

      Gtk.Label.Gtk_New
        (Update_networks_Set.Lab,
         To_String (Lb_Networks_Name (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_networks_Set.network_Table,
         Update_networks_Set.Lab,
         0,
         1,
         0,
         1);

      Gtk.Label.Gtk_New
        (Update_networks_Set.Lab,
         To_String (Lb_Protocol (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_networks_Set.network_Table,
         Update_networks_Set.Lab,
         0,
         1,
         1,
         2);

      Gtk.GEntry.Gtk_New (Update_networks_Set.Name, 50);
      Set_Editable (Update_networks_Set.Name, True);
      Attach_Defaults
        (Update_networks_Set.network_Table,
         Update_networks_Set.Name,
         1,
         2,
         0,
         1);

      Gtk.Option_Menu.Gtk_New (Update_networks_Set.protocol_Menu);
      Build_Option_Menu
        (Update_networks_Set.protocol_Menu,
         Update_networks_Set.Clist_protocol_Menu,
         Items,
         -1,
         null);
      Attach_Defaults
        (Update_networks_Set.network_Table,
         Update_networks_Set.protocol_Menu,
         1,
         2,
         1,
         2);

      Gtk.Button.Gtk_New
        (Update_networks_Set.Close_Button,
         To_String (Lb_Close (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_networks_Set.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_networks_Set.Advanced_Button,
         To_String (Lb_Advanced (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_networks_Set.Delete_Button,
         To_String (lb_delete (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_networks_Set.Modify_Button,
         To_String (Lb_Modify (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_networks_Set.Add_Button,
         To_String (Lb_add (Current_Language)));

      Add
        (Update_networks_Set.Hbutton_Box_Modify,
         Update_networks_Set.Close_Button);
      Add
        (Update_networks_Set.Hbutton_Box_Modify,
         Update_networks_Set.Cancel_Button);
      Add
        (Update_networks_Set.Hbutton_Box_Modify,
         Update_networks_Set.Advanced_Button);
      Add
        (Update_networks_Set.Hbutton_Box_Modify,
         Update_networks_Set.Delete_Button);
      Add
        (Update_networks_Set.Hbutton_Box_Modify,
         Update_networks_Set.Modify_Button);
      Add
        (Update_networks_Set.Hbutton_Box_Modify,
         Update_networks_Set.Add_Button);

      -- Spreadsheet for network information
      --
      Gtk.Scrolled_Window.Gtk_New
        (Update_networks_Set.Scroll_List,
         null,
         null);
      Add (Update_networks_Set.Vbox_List, Update_networks_Set.Scroll_List);

      Update_networks_Set.List_Titles :=
        To_String (Lb_Name (Current_Language)) +
        To_String (Lb_Protocol (Current_Language));

      Gtk.Clist.Gtk_New
        (Update_networks_Set.networks_List,
         2,
         Update_networks_Set.List_Titles);
      Set_USize (Update_networks_Set.networks_List, 475, 200);
      Set_Sort_Column (Update_networks_Set.networks_List, 0);
      Set_Sort_Type (Update_networks_Set.networks_List, Ascending);
      Set_Auto_Sort (Update_networks_Set.networks_List, True);
      Add
        (Update_networks_Set.Scroll_List,
         Update_networks_Set.networks_List);

      Show_networks (Update_networks_Set);

      -- Select network from spreadsheet activation
      --
      List_Callback.Connect
        (Update_networks_Set.networks_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'Access));

      Button_Callback.Connect
        (Update_networks_Set.Modify_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'Access));
      Button_Callback.Connect
        (Update_networks_Set.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));
      Button_Callback.Connect
        (Update_networks_Set.Advanced_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Advanced_Pressed'Access));
      Button_Callback.Connect
        (Update_networks_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));
      Button_Callback.Connect
        (Update_networks_Set.Add_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'Access));
      Button_Callback.Connect
        (Update_networks_Set.Delete_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'Access));

   end Initialize;

end Update_networks_Set_Pkg;
