------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
generic
   type Element is private;
   with procedure Put (E : in Element);

package Binary_Trees is

   No_Cell : exception;

   type Cell is private;
   type Binary_Tree is access Cell;

   procedure Add_Left (L : in out Binary_Tree; E : in Element);
   procedure Add_Right (L : in out Binary_Tree; E : in Element);

   function Get_Left (L : in Binary_Tree) return Binary_Tree;
   function Get_Right (L : in Binary_Tree) return Binary_Tree;

   procedure Put (L : in Binary_Tree);

   procedure Delete (L : in out Binary_Tree; E : in Element);

   procedure Duplicate (Src : in Binary_Tree; Dest : in out Binary_Tree);

private

   type Cell is record
      Left, Right : Binary_Tree;
      Info        : Element;
   end record;

end Binary_Trees;
