------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Ada.Numerics.Aux;       use Ada.Numerics.Aux;
with Unchecked_Deallocation;

package Double_Util is

   -- Pointer to a double
   --
   type Double_Ptr is access Double;

   procedure Free is new Unchecked_Deallocation (Double, Double_Ptr);

   -- Initialize a double
   --
   procedure Initialize (F : out Double);

   -- Convert a double into a string
   --
   function Format
     (F     : in Double;
      After : in Integer := 5)
      return  Unbounded_String;

   -- Display a double
   --
   procedure Put (F : in Double);
   procedure Put (F : in Double_Ptr);
   procedure Put_Line (F : in Double);

   -- New equal function for numeric stability
   --
   function Equal
     (Num1, Num2 : in Double;
      Accuracy   : in Integer := 10000)
      return       Boolean;

   -- return A Modulo B
   --
   function My_Mod (A : Double; B : Double) return Double;

end Double_Util;
