------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
generic

   Max_Element : Natural := 100;
   type Element is private;
   with procedure Put (A_Element : in Element);

package Fifos is

   type Fifo is tagged private;
   type Element_Range is new Natural range 0 .. Max_Element;
   type Fifo_Ptr is access all Fifo'Class;

   Full_Fifo : exception;
   Empty_Fifo : exception;
   Element_Not_Found : exception;

   -- Clear the fifo
   --
   procedure Reset (My_Fifo : in out Fifo);

   -- Put the content of the fifo in the
   -- screen
   --
   procedure Put (My_Fifo : in Fifo);

   -- Insert a element into the fifo
   -- We insert with the head pointer
   --
   procedure Insert (My_Fifo : in out Fifo; E : in Element);

   -- Extract a element from the fifo
   -- We extract with the queue pointer
   --
   procedure Extract (My_Fifo : in out Fifo; E : in out Element);

   -- Extract "Element" from any where
   --
   procedure Extract_Any_Where (My_Fifo : in out Fifo; E : in Element);

   -- Read (but not extract) a element from the fifo
   -- We read with the queue pointer
   --
   function Consult (My_Fifo : in Fifo) return Element;

   -- Get the number of elements currently present
   -- in the fifo
   --
   function Get_Size (My_Fifo : in Fifo) return Natural;

   -- Is my_fifo empty ?
   --
   function Is_Empty (My_Fifo : in Fifo) return Boolean;

private

   type Element_Table is array (Element_Range) of Element;

   type Fifo is tagged record
      Size  : Natural := 0;
      Data  : Element_Table;
      Queue : Element_Range;
      Head  : Element_Range;
   end record;

end Fifos;
