------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Text_IO; use Text_IO;

package body Hashs is

   procedure Put (H : in Hash) is

      Current : Link;

   begin
      for I in Hash'Range loop
         Current := H (I);
         while Current /= null loop
            Put ("Index = " & To_String (Current.Index));
            Put ("   Data = ");
            Put (Current.A_Data);
            New_Line;
            New_Line;

            Current := Current.Next;
         end loop;
      end loop;
   end Put;

   procedure Initialize (H : in out Hash) is
   begin
      for I in Hash'Range loop
         H (I) := null;
      end loop;
   end Initialize;

   procedure Add
     (H     : in out Hash;
      Index : in Unbounded_String;
      D     : in Data_Type)
   is

      Item : constant Natural := Compute_Hash (Index);
      C    : Link;

   begin
      if H (Item) /= null then
         C             := new Cell;
         C.A_Data      := D;
         C.Index       := Index;
         C.Next        := H (Item).Next;
         H (Item).Next := C;
      else
         H (Item)        := new Cell;
         H (Item).A_Data := D;
         H (Item).Index  := Index;
      end if;
   end Add;

   function Consult
     (H     : in Hash;
      Index : in Unbounded_String)
      return  Data_Type
   is
      Item    : constant Natural := Compute_Hash (Index);
      Current : Link             := H (Item);

   begin

      while Current /= null loop
         if Current.Index = Index then
            return Current.A_Data;
         end if;
         Current := Current.Next;
      end loop;

      raise Data_Not_Found;

   end Consult;

end Hashs;
