------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Text_IO; use Text_IO;

package body Indexed_Tables is

   procedure Initialize (A_Table : out Indexed_Table) is
   begin
      A_Table.Nb_Entries := 0;
      for I in Indexed_Table_Range loop
         Initialize (A_Table.Entries (I).Data);
      end loop;
   end Initialize;

   procedure Put (A_Item : in Item) is
   begin
      Put (A_Item.Item);
      Put ("/");
      Put (A_Item.Data);
      New_Line;
   end Put;

   procedure Put (A_Table : in Indexed_Table) is
   begin
      Put_Line (" Nb entries : " & A_Table.Nb_Entries'Img);
      for I in First_Range .. First_Range + A_Table.Nb_Entries - 1 loop
         Put (A_Table.Entries (I));
      end loop;
   end Put;

   procedure Delete
     (A_Table : in out Indexed_Table;
      A_Item  : Indexed_Table_Range)
   is
   begin

      A_Table.Entries (A_Item) := A_Table.Entries (A_Table.Nb_Entries - 1);
      A_Table.Nb_Entries       := A_Table.Nb_Entries - 1;

   end Delete;

   procedure Add
     (A_Table   : in out Indexed_Table;
      A_Index   : Index;
      A_Element : Element)
   is
   begin

      if A_Table.Nb_Entries > Indexed_Table_Range'Last - 1 then
         raise Indexed_Table_Full;
      end if;

      A_Table.Entries (A_Table.Nb_Entries).Item := A_Index;
      A_Table.Entries (A_Table.Nb_Entries).Data := A_Element;
      A_Table.Nb_Entries                        := A_Table.Nb_Entries + 1;
   end Add;

end Indexed_Tables;
