------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Text_IO; use Text_IO;
with Ada.Sequential_IO;

package body Io_Tools is

   -- Check is a file can be read
   -- Return True if the file can be read
   --
   function Can_Be_Read (File_Name : Unbounded_String) return Boolean is
      A_File : File_Type;
      Result : Boolean := True;

   begin

      begin
         Open (A_File, In_File, To_String (File_Name));
         Close (A_File);

      exception
         when others =>
            Result := False;
      end;

      return Result;

   end Can_Be_Read;

   procedure Reraise;
   pragma No_Return (Reraise);
   pragma Import (C, Reraise, "__gnat_reraise");
   --  Reraises the exception referenced by the Current_Excep field of
   --  the TSD (all fields of this exception occurrence are set). Abort
   --  is deferred before the reraise operation.

   procedure Write_Sequential_File
     (File_Name : in Unbounded_String;
      Data      : in Unbounded_String)
   is

      package Set_File is new Ada.Sequential_IO (Character);
      use Set_File;
      A_File : Set_File.File_Type;

   begin
      Create (A_File, Mode => Out_File, Name => To_String (File_Name));

      if not Is_Open (A_File) then
         raise Can_Not_Open_File_For_Writing;
      end if;

      for I in 1 .. Length (Data) loop
         Write (A_File, Element (Data, I));
      end loop;

      Close (A_File);

   exception
      when others =>
         Close (A_File);
         Reraise;

   end Write_Sequential_File;

   procedure Write_Sequential_File
     (File_Name : in String;
      Data      : in Unbounded_String)
   is
   begin
      Write_Sequential_File (To_Unbounded_String (File_Name), Data);
   end Write_Sequential_File;

   function Read_Sequential_File
     (File_Name : String)
      return      Unbounded_String
   is
   begin
      return Read_Sequential_File (To_Unbounded_String (File_Name));
   end Read_Sequential_File;

   function Read_Sequential_File
     (File_Name : Unbounded_String)
      return      Unbounded_String
   is
      Desc   : Character;
      Result : Unbounded_String;

      package Set_File is new Ada.Sequential_IO (Character);
      use Set_File;
      A_File : Set_File.File_Type;

   begin
      Open (A_File, Mode => In_File, Name => To_String (File_Name));

      if not Is_Open (A_File) then
         raise Can_Not_Open_File_For_Reading;
      end if;

      while not End_Of_File (A_File) loop
         Read (A_File, Desc);
         Result := Result & Desc;
      end loop;

      Close (A_File);

      return Result;

   exception
      when others =>
         Close (A_File);
         Reraise;

   end Read_Sequential_File;

end Io_Tools;
