------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

package Io_Tools is

   type File_Mode is (Save_In_New_File, Append_To_Old_File);

   Can_Not_Open_File_For_Reading : exception;
   Can_Not_Open_File_For_Writing : exception;

   -- Read a ASCII file and store it in a
   -- ADA string
   -- I/O Exceptions can be raised in case of I/O error
   --
   function Read_Sequential_File
     (File_Name : Unbounded_String)
      return      Unbounded_String;
   function Read_Sequential_File
     (File_Name : String)
      return      Unbounded_String;

   -- Read a ASCII file and store it in a
   -- ADA string
   -- I/O Exceptions can be raised in case of I/O error
   --
   procedure Write_Sequential_File
     (File_Name : in Unbounded_String;
      Data      : in Unbounded_String);
   procedure Write_Sequential_File
     (File_Name : in String;
      Data      : in Unbounded_String);

   -- Check is a file can be read
   -- Return True if the file can be read
   --
   function Can_Be_Read (File_Name : Unbounded_String) return Boolean;

end Io_Tools;
