------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Text_IO; use Text_IO;

package body Real_Util is

   package Real_Element_Io is new Text_IO.Float_IO (Real_Element);

   function Equal
     (Num1, Num2 : Real_Element;
      Accuracy   : Integer := 10000)
      return       Boolean
   is
      My_Epsi : constant Real_Element := 1.0 / Real_Element (Accuracy);
      Diff    : constant Real_Element := abs (Num1 - Num2);
   begin
      return (Diff < My_Epsi);
   end Equal;

   function Format
     (F     : in Real_Element;
      After : in Natural := 5)
      return  Unbounded_String
   is
      S      : String (1 .. 10000);
      Result : Unbounded_String;
   begin
      Real_Element_Io.Put (S, F, After, Exp => 0);

      -- suppress blank
      for I in 1 .. S'Length loop
         if S (I) /= ' ' then
            Append (Result, S (I));
         end if;
      end loop;

      return (Result);
   end Format;

end Real_Util;
