------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a GNU GPL real time scheduling analysis tool.
-- This program provides services to automatically check performances
-- of real time architectures.
--
-- Copyright (C) 2002-2010,  by Frank Singhoff, Alain Plantec, Jerome Legrand
--
-- The Cheddar project was started in 2002 by
-- the LISyC Team, University of Western Britanny.
--
-- Since 2008, Ellidiss technologies also contributes to the development of
-- Cheddar and provides industrial support.
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : cheddar@listes.univ-brest.fr
--
-----------------------------------------------------------------------------
-- Last update :
--    $Rev: 209 $
--    $Date: 2008-06-29 14:47:31 +0200 (dim, 29 jun 2008) $
--    $Author: singhoff $
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Ada.Strings.Maps.Constants;
with Text_IO; use Text_IO;

package body Unbounded_Strings is

   function To_Upper (S : String) return Unbounded_String is
   begin
      return To_Upper (To_Unbounded_String (S));
   end To_Upper;

   function To_Lower (S : String) return Unbounded_String is
   begin
      return To_Lower (To_Unbounded_String (S));
   end To_Lower;

   function To_Upper (S : Unbounded_String) return Unbounded_String is
   begin
      return Translate (S, Ada.Strings.Maps.Constants.Upper_Case_Map);
   end To_Upper;

   function To_Lower (S : Unbounded_String) return Unbounded_String is
   begin
      return Translate (S, Ada.Strings.Maps.Constants.Lower_Case_Map);
   end To_Lower;

   procedure To_Double
     (From : in Unbounded_String;
      To   : out Double;
      Ok   : out Boolean)
   is
   begin
      To := Double'Value (To_String (From));
      Ok := True;
   exception
      when Constraint_Error =>
         To := 0.0;
         Ok := False;
   end To_Double;

   procedure Initialize (St : out Unbounded_String) is
   begin
      St := Empty_String;
   end Initialize;

   procedure Put (S : Unbounded_String_Ptr) is
   begin
      Put (To_String (S.all));
   end Put;

   function Suppress_Space (S : in String) return Unbounded_String is
      Result : Unbounded_String;
   begin
      for I in 1 .. S'Length loop
         if S (I) /= ' ' then
            Append (Result, S (I));
         end if;
      end loop;
      return Result;
   end Suppress_Space;

   function Suppress_Space
     (S    : in Unbounded_String)
      return Unbounded_String
   is
   begin
      return Suppress_Space (To_String (S));
   end Suppress_Space;

   function Has_Space (S : in String) return Boolean is
   begin
      for I in 1 .. S'Length loop
         if S (I) /= ' ' then
            return True;
         end if;
      end loop;
      return False;
   end Has_Space;

   function Has_Space (S : in Unbounded_String) return Boolean is
   begin
      return Has_Space (To_String (S));
   end Has_Space;

end Unbounded_Strings;
