
--******************************************************************************
--                                                                            **
--  File:        NEWTON.ADB                                                   **
--  Description: Calculate the number PI                                      **
--  Revision:    1.23                                                         **
--  Date:        February 3, 2001                                             **
--  Author:      Jrme Delcourt                                              **
--  Mail:        sikander@club-internet.fr                                    **
--                                                                            **
--  Copyright (c) Jrme Delcourt, 1998, 1999, 2000, 2001                     **
--  62, rue Ngrier                                                           **
--  59800 Lille                                                               **
--  FRANCE                                                                    **
--                                                                            **
--  Permission granted to use for any purpose, provided this copyright        **
--  remains attached and unmodified.                                          **
--                                                                            **
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR            **
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED            **
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.       **
--                                                                            **
--******************************************************************************

WITH ModInt;           USE ModInt;
WITH TEXT_IO;          USE TEXT_IO;
WITH Index_IO;         USE Index_IO;
WITH My_Big;           USE My_Big;
                       USE My_Big.Unsigned_Number;
                       USE My_Big.Big_Fraction;
WITH Utils_Pi;         USE Utils_Pi;


PROCEDURE Newton IS

-- PI/6 = Arcsin(1/2)

--****************************************************************************** 
-- Declarations of variables
--****************************************************************************** 

  Borne_Inf, Borne_Sup : My_Index;
  Result, TempFract : Fraction;

--****************************************************************************** 
-- Declarations of functions
--****************************************************************************** 

PROCEDURE ArcSin(Result, Temp_Fract1 : IN OUT Fraction;
                 Loop_Inf, Loop_Sup : IN My_Index) IS
  Temp, Two_I_plus_1 : Big_Unsigned;
  Coeff : Big_Unsigned := (Big_Unsigned_Two ** 2) ** Loop_Inf;
  Temp_Result : Fraction := CreateFraction(Big_Unsigned_Zero, Big_Unsigned_One);
    -- Temp_Result = 0/1 = 0
  Temp_Fract2 : Fraction := Fract_One;  -- Temp_Fract = 1
BEGIN
  FOR I IN Loop_Inf..Loop_Sup LOOP
    Two_I_plus_1 := Convert(I) * 2;
    IF (I >= 2) THEN
      Temp := Two_I_plus_1;
      Temp_Fract2 := Temp_Fract2 / Temp;
      Dec(Temp);
      Temp_Fract2 := Temp_Fract2 * Temp;
    END IF;
    Inc(Two_I_plus_1);
    Coeff := Coeff * 4;     -- Now, Coeff := 2 ** (2*I + 1)
    Temp_Result := Temp_Result + (Temp_Fract2 / Two_I_plus_1) / Coeff;
  END LOOP;
  --
  Result := Result + Temp_Result * Temp_Fract1;
  Temp_Fract1 := Temp_Fract1 * Temp_Fract2;
END ArcSin;

--******************************************************************************
--******************************************************************************

BEGIN
  -- Initialisation
  Init(Borne_Inf, Borne_Sup, Result, TempFract);
  -- Computation
  ArcSin(Result, TempFract, Borne_Inf, Borne_Sup);
  -- Save the result
  Save(Borne_Sup, Result, TempFract);
END Newton;
