
--******************************************************************************
--                                                                            **
--  File:        NEWTON2.ADB                                                  **
--  Description: Calculate the number PI                                      **
--  Revision:    1.23                                                         **
--  Date:        February 3, 2001                                             **
--  Author:      Jrme Delcourt                                              **
--  Mail:        sikander@club-internet.fr                                    **
--                                                                            **
--  Copyright (c) Jrme Delcourt, 1998, 1999, 2000, 2001                     **
--  62, rue Ngrier                                                           **
--  59800 Lille                                                               **
--  FRANCE                                                                    **
--                                                                            **
--  Permission granted to use for any purpose, provided this copyright        **
--  remains attached and unmodified.                                          **
--                                                                            **
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR            **
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED            **
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.       **
--                                                                            **
--******************************************************************************

WITH ModInt;           USE ModInt;
WITH TEXT_IO;          USE TEXT_IO;
WITH PC_Keyboard;      USE PC_Keyboard;
WITH My_Big;           USE My_Big;
                       USE My_Big.Unsigned_Number;
                       USE My_Big.Big_Fraction;
WITH Index_IO;         USE Index_IO;
WITH Utils_Pi;         USE Utils_Pi;


PROCEDURE Newton2 IS

-- PI/6 = Arcsin(1/2)

--****************************************************************************** 
-- Declarations of variables
--****************************************************************************** 

  Loop_Inf, Loop_Sup : My_Index;
  Result, TempFract : Fraction;

--****************************************************************************** 
-- Declarations of functions
--****************************************************************************** 

PROCEDURE Calc(Inf : IN My_Index; Sup : OUT My_Index;
               ArcSin, Temp_Fract1 : IN OUT Fraction) IS
  I : My_Index := Inf;
  Temp, Two_I_plus_1 : Big_Unsigned;
  Coeff : Big_Unsigned := Big_Unsigned_Two ** (2*Inf);
  Temp_Result : Fraction := CreateFraction(Big_Unsigned_Zero, Big_Unsigned_One);
    -- Temp_Result = 0/1 = 0
  Temp_Fract2 : Fraction := Fract_One;  -- Temp_Result = 1
BEGIN
  LOOP
    Two_I_plus_1 := Convert(I) * 2;
    IF (I >= 2) THEN
      Temp := Two_I_plus_1;
      Temp_Fract2 := Temp_Fract2 / Temp;
      Dec(Temp);
      Temp_Fract2 := Temp_Fract2 * Temp;
    END IF;
    Inc(Two_I_plus_1);
    Coeff := Coeff * 4;     -- Now, Coeff := 2 ** (2*I + 1);
    Temp_Result := Temp_Result + (Temp_Fract2 / Two_I_plus_1) / Coeff;
    IF (Is_Key_Waiting = TRUE) OR ELSE (I = My_Index'LAST) THEN
      Sup := I;
      EXIT;
    END IF;
    I := I + 1;
  END LOOP;
  --
  ArcSin := ArcSin + Temp_Result * Temp_Fract1;
  Temp_Fract1 := Temp_Fract1 * Temp_Fract2;
  Clear_Keyboard_Buffer;
EXCEPTION
  WHEN OTHERS => PUT("Problem with loop n");
                 PUT(I, WIDTH => 0);
                 RAISE;
END Calc;

--******************************************************************************
--******************************************************************************

BEGIN
  -- Initialisation
  Init(Loop_Inf, Result, TempFract);
  -- Computation
  Calc(Loop_Inf, Loop_Sup, Result, TempFract);
  -- Save the result
  Save(Loop_Sup, Result, TempFract);
END Newton2;
