
--******************************************************************************
-----------------------------------------------------------------------
--
--  File:        pckeyboa.adb
--  Description: Basic keyboard interface
--  Rev:         0.7
--  Date:        01-feb-98
--  Author:      Jerry van Dijk
--  Mail:        jdijk@acm.org
--
--  Copyright (c) Jerry van Dijk, 1996, 1997, 1998
--  Billie Holidaystraat 28
--  2324 LK Leiden
--  THE NETHERLANDS
--  tel int + 31 71 531 4365
--
--  Permission granted to use for any purpose, provided this copyright
--  remains attached and unmodified.
--
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--
-----------------------------------------------------------------------


   ------------------------------------------------------------
   --                       PACKAGE CONTAINS                 --
   ------------------------------------------------------------
   -- Types:                                                 --
   --    Keyboard_Scancode - Scancode of a keypress          --
   --                                                        --
   -- Functions:                                             --
   --    Is_Key_Waiting    - Check if a key has been pressed --
   --    Get_Scancode      - Waits for a key press           --
   --    Get_Char          - Waits for a key press           --
   ------------------------------------------------------------


with Interfaces;    use Interfaces;
with DJGPP_Library; use DJGPP_Library;

package body PC_Keyboard is

   ----------------------------------------------------
   -- NAME:    Is_Key_Waiting                        --
   --                                                --
   -- PURPOSE: Check if a key has been pressed       --
   --                                                --
   -- RETURNS: True if key pressed, False otherwise  --
   --                                                --
   -- NOTES:   1. Leaves keypress in keyboard buffer --
   ----------------------------------------------------

   function Is_Key_Waiting return Boolean is
      Regs   : Dpmi_Regs;
      Result : Boolean;
   begin
      Regs.Ax := 16#1100#;
      Dpmi_Int(16#16#, Regs);
      Result := ((Regs.Flags and 16#40#) = 0);
      return Result;
   end Is_Key_Waiting;


   -------------------------------------------
   -- NAME:    Get_Scancode                 --
   --                                       --
   -- PURPOSE: Waits for a key press        --
   --                                       --
   -- RETURNS: Key Scancode                 --
   --                                       --
   -- NOTES:   1. Control-Break not noticed --
   -------------------------------------------
   function Get_Scancode return Keyboard_Scancode is
      Regs : Dpmi_Regs;
   begin
      Regs.Ax := 16#0000#;
      Dpmi_Int(16#16#, Regs);
      return Keyboard_Scancode(Regs.Ax);
   end Get_Scancode;


   --------------------------------------------------
   -- NAME:    Get_Char                            --
   --                                              --
   -- PURPOSE: Waits for a key press               --
   --                                              --
   -- RETURNS: Character pressed, NUL if not ASCII --
   --                                              --
   -- NOTES:   1. Control-Break not noticed        --
   --------------------------------------------------
   function Get_Char return Character is
   begin
      return Character'VAL(Unsigned_16(Get_Scancode) and 16#00FF#);
   end Get_Char;


   --------------------------------------------------
   -- NAME:    Clear_Keyboard_Buffer               --
   --                                              --
   -- PURPOSE: Clear the keyboard buffer           --
   --                                              --
   -- RETURNS: None                                --
   --                                              --
   -- NOTES:   None                                --
   --------------------------------------------------
   procedure Clear_Keyboard_Buffer is
     Code : Keyboard_Scancode;
   begin
     while (Is_Key_Waiting = TRUE) loop
       Code := Get_Scancode;
     end loop;
   end Clear_Keyboard_Buffer;
 
end PC_Keyboard;
