
--******************************************************************************
--                                                                            **
--  File:        UTILS_PI.ADB                                                 **
--  Description: Package body of the package Utils_Pi                         **
--  Revision:    1.23                                                         **
--  Date:        February 3, 2001                                             **
--  Author:      Jrme Delcourt                                              **
--  Mail:        sikander@club-internet.fr                                    **
--                                                                            **
--  Copyright (c) Jrme Delcourt, 1998, 1999, 2000, 2001                     **
--  62, rue Ngrier                                                           **
--  59800 Lille                                                               **
--  FRANCE                                                                    **
--                                                                            **
--  Permission granted to use for any purpose, provided this copyright        **
--  remains attached and unmodified.                                          **
--                                                                            **
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR            **
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED            **
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.       **
--                                                                            **
--******************************************************************************

PACKAGE BODY Utils_Pi IS

--****************************************************************************** 
--****************************************************************************** 

  Loop_FileName : CONSTANT FileName := "CALCLOOP.NWT";
  Result_FileName : CONSTANT FileName := "RESULTAT.NWT";
  Temp_Fract_FileName : CONSTANT FileName := "TEMPFRAC.NWT";

--****************************************************************************** 
-- Declarations of functions
--****************************************************************************** 

FUNCTION Files_Exist RETURN BOOLEAN IS
  Temp_Result : BOOLEAN;
BEGIN
  Temp_Result := Does_File_Exist(Loop_FileName)
                 AND THEN Does_File_Exist(Result_FileName)
                 AND THEN Does_File_Exist(Temp_Fract_FileName);
  RETURN Temp_Result;
END Files_Exist;

--****************************************************************************** 
--****************************************************************************** 

PROCEDURE Init(Inf, Sup : OUT My_Index; Old_ArcSin, Temp_Fract : OUT Fraction) IS
  File_NbLoop : Index_Direct_IO.FILE_TYPE;
  File_Result : Fraction_Direct_IO.FILE_TYPE;
  Nb_Loop, Old_Loop : My_Index;
BEGIN
  IF Files_Exist THEN
    OPEN(File_NbLoop, IN_FILE, Loop_FileName);
    READ(File_NbLoop, Old_Loop);
    CLOSE(File_NbLoop);
    OPEN(File_Result, IN_FILE, Result_FileName);
    READ(File_Result, Old_ArcSin);
    CLOSE(File_Result);
    OPEN(File_Result, IN_FILE, Temp_Fract_FileName);
    READ(File_Result, Temp_Fract);
    CLOSE(File_Result);
  ELSE
    Old_ArcSin := CreateFraction(Convert(1), Convert(2), TRUE);
    Temp_Fract := Fract_One;
    Old_Loop := 0;
  END IF;
  PUT("Number of loops already done : ");
  PUT(Old_Loop, WIDTH => 0);
  NEW_LINE;
  PUT("Number of loops to do ? ");
  GET(Nb_Loop);
  Inf := Old_Loop + 1;
  Sup := Old_Loop + Nb_Loop;
END Init;

--******************************************************************************
--******************************************************************************

PROCEDURE Init(Inf : OUT My_Index; Old_ArcSin, Temp_Fract : OUT Fraction) IS
  File_NbLoop : Index_Direct_IO.FILE_TYPE;
  File_Result : Fraction_Direct_IO.FILE_TYPE;
  Old_Loop : My_Index;
BEGIN
  IF Files_Exist THEN
    OPEN(File_NbLoop, IN_FILE, Loop_FileName);
    READ(File_NbLoop, Old_Loop);
    CLOSE(File_NbLoop);
    OPEN(File_Result, IN_FILE, Result_FileName);
    READ(File_Result, Old_ArcSin);
    CLOSE(File_Result);
    OPEN(File_Result, IN_FILE, Temp_Fract_FileName);
    READ(File_Result, Temp_Fract);
    CLOSE(File_Result);
  ELSE
    Old_ArcSin := CreateFraction(Convert(1), Convert(2), TRUE);
    Temp_Fract := Fract_One;
    Old_Loop := 0;
  END IF;
  PUT("Number of loops already done : ");
  PUT(Old_Loop, WIDTH => 0);
  NEW_LINE;
  Inf := Old_Loop + 1;
END Init;

--******************************************************************************
--******************************************************************************

PROCEDURE Save(Sup : IN My_Index; Old_Result, Temp_Fract : IN Fraction) IS
  File_NbLoop : Index_Direct_IO.FILE_TYPE;
  File_Result : Fraction_Direct_IO.FILE_TYPE;
BEGIN
  CREATE(File_NbLoop, OUT_FILE, Loop_FileName);
  WRITE(File_NbLoop, Sup);
  CLOSE(File_NbLoop);
  CREATE(File_Result, OUT_FILE, Result_FileName);
  WRITE(File_Result, Old_Result);
  CLOSE(File_Result);
  CREATE(File_Result, OUT_FILE, Temp_Fract_FileName);
  WRITE(File_Result, Temp_Fract);
  CLOSE(File_Result);
END Save;

--******************************************************************************
--******************************************************************************

FUNCTION Result2UString(Result : IN Fraction;
                        Nb_Digits : IN My_Index) RETURN UString IS
  --
  -- PI/6 = ArcSin(1/2)
  -- So PI # 6 * Result
  --
  Exponent : CONSTANT Big_Unsigned := Convert(10) ** Nb_Digits;
  Temp_3 : CONSTANT Big_Unsigned := Exponent * 3;
  Temp_Result1 : CONSTANT Big_Unsigned
        := ((Get_Numerator(Result)*Exponent*6)/Get_Denominator(Result))-Temp_3;
  Temp_Result2 : UString;
BEGIN
  Temp_Result2 := "3." & Big_Unsigned2UString(Temp_Result1);
  RETURN Temp_Result2;
END Result2UString;

--******************************************************************************
--******************************************************************************

PROCEDURE Resultat(Result : IN Fraction; BorneSup : IN My_Index) IS
  Char : CHARACTER := 'Z';
  Fichier : TEXT_IO.FILE_TYPE;
  Nb_Digits : My_Index;
  Nb_Boucles : CONSTANT STRING(1..18) := "Number of loops : ";
BEGIN
  WHILE (Char /= 'Y') AND THEN (Char /= 'y') AND THEN
        (Char /= 'N') AND THEN (Char /= 'n') LOOP
    PUT("Do you want the output of the result in a file ? ");
    GET(Char);
  END LOOP;
  PUT("Number of decimal digits ? ");
  GET(Nb_Digits);

  IF (Char = 'N') OR ELSE (Char = 'n') THEN
    PUT_LINE(Result2UString(Result, Nb_Digits));
  ELSE
    CREATE(Fichier, OUT_FILE, "RESULT.TXT");
    PUT_LINE(Fichier, Result2UString(Result, Nb_Digits));
    PUT(Fichier, Nb_Boucles);
    PUT(Fichier, BorneSup, WIDTH => 0);
    NEW_LINE(Fichier);
    CLOSE(Fichier);
  END IF;
END Resultat;

--******************************************************************************
--******************************************************************************

END Utils_Pi;
