--******************************************************************************
--                                                                            **
--  File:        BINUSINU.ADB                                                 **
--  Description: Big_Number.Signed_Number package body                        **
--  Revision:    0.28 (beta version)                                          **
--  Date:        July 8, 2001                                                 **
--  Author:      Jrme Delcourt                                              **
--  Mail:        sikander@club-internet.fr                                    **
--                                                                            **
--  Copyright (c) Jrme Delcourt, 1998, 1999, 2000, 2001                     **
--  62, rue Ngrier                                                           **
--  59800 Lille                                                               **
--  FRANCE                                                                    **
--                                                                            **
--  Permission granted to use for any purpose, provided this copyright        **
--  remains attached and unmodified.                                          **
--                                                                            **
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR            **
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED            **
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.       **
--                                                                            **
--******************************************************************************

SEPARATE(Big_Number)

PACKAGE BODY Signed_Number IS

--******************************************************************************
--******************************************************************************
--**                              Comparisons                                 **
--******************************************************************************
--******************************************************************************

FUNCTION "<"(Left, Right : IN Big_Signed) RETURN BOOLEAN IS
BEGIN
  IF (Left.Positive /= Right.Positive) THEN
    RETURN BOOLEAN(Right.Positive);
  ELSIF (Left = Right) THEN
    RETURN FALSE;
  ELSE
    -- Now, x and y have the same sign and are different.
    DECLARE
      Indice : Index_Type := Left.Last_Significant_Digit;
    BEGIN
      -- Left and Right have the same sign but their mantissas are different.
      IF (Indice /= Right.Last_Significant_Digit) THEN
        RETURN ((Indice < Right.Last_Significant_Digit) = Left.Positive);
      END IF;
      WHILE (Left.Mantissa(Indice) = Right.Mantissa(Indice))
            AND THEN (Indice > First_Indice) LOOP
        Indice := Indice - 1;
      END LOOP;
      RETURN ((Left.Mantissa(Indice) < Right.Mantissa(Indice)) = Left.Positive);
    END;
  END IF;
END "<";

--******************************************************************************

FUNCTION ">"(Left, Right : IN Big_Signed) RETURN BOOLEAN IS
  -- The comparison "Left > Right" is equivalent to "Right < Left"...
BEGIN
  RETURN Right < Left;
END ">";

--******************************************************************************

FUNCTION "<="(Left, Right : IN Big_Signed) RETURN BOOLEAN IS
BEGIN
  RETURN NOT (Right < Left);
END "<=";

--******************************************************************************

FUNCTION ">="(Left, Right : IN Big_Signed) RETURN BOOLEAN IS
  -- The comparison "Left >= Right" is the same test than "Right <= Left"...
BEGIN
  RETURN Right <= Left;
END ">=";

--******************************************************************************

FUNCTION Is_Null(x : IN Big_Signed) RETURN BOOLEAN IS
BEGIN
  IF (x.Last_Significant_Digit = First_Indice) AND THEN
     (x.Mantissa(First_Indice) = 0) THEN
    RETURN TRUE;
  ELSE
    RETURN FALSE;
  END IF;
END Is_Null;

--******************************************************************************

FUNCTION Min(x, y : IN Big_Signed) RETURN Big_Signed IS
BEGIN
  IF (x < y) THEN
    RETURN x;
  ELSE
    RETURN y;
  END IF;
END Min;

--******************************************************************************

FUNCTION Max(x, y : IN Big_Signed) RETURN Big_Signed IS
BEGIN
  IF (x < y) THEN
    RETURN y;
  ELSE
    RETURN x;
  END IF;
END Max;

--******************************************************************************
--******************************************************************************
--**                            Binary operations                             **
--******************************************************************************
--******************************************************************************

PROCEDURE Swap1(x, y : IN OUT Big_Signed) IS
  Temp : CONSTANT Big_Signed := x;
BEGIN
  x := y;
  y := Temp;
END Swap1;

--******************************************************************************

PROCEDURE Swap2(x, y : IN OUT Big_Signed) IS
  Max_Index : CONSTANT Index_Type
                     := Max(x.Last_Significant_Digit, y.Last_Significant_Digit);
  Temp : My_Type;
  Temp_Positive : CONSTANT BOOLEAN := x.Positive;
BEGIN
  x.Positive := y.Positive;
  y.Positive := Temp_Positive;
  FOR I IN First_Indice..Max_Index LOOP
    Temp := x.Mantissa(I);
    x.Mantissa(I) := y.Mantissa(I);
    y.Mantissa(I) := Temp;
  END LOOP;
END Swap2;

--******************************************************************************

PROCEDURE Inc(x : IN OUT Big_Signed) IS
  Temp_x : Big_Unsigned := Signed2Unsigned(x);
BEGIN
  IF (x.Last_Significant_Digit = First_Indice) AND THEN
     (x.Mantissa(First_Indice) = 0) THEN
     -- If x = 0...
    x := Big_Signed_One;
    RETURN;
  ELSIF (x.Positive = TRUE) THEN
    -- x > 0
    Inc(Temp_x);
    x := Unsigned2Signed(Temp_x, x.Positive);
  ELSE
    -- x < 0
    IF (x.Last_Significant_Digit = First_Indice) AND THEN
       (x.Mantissa(First_Indice) = 1) THEN
      x := Big_Signed_Zero;
    ELSE
      Dec(Temp_x);
      x := Unsigned2Signed(Temp_x, x.Positive);
    END IF;
  END IF;
END Inc;

--******************************************************************************

PROCEDURE Dec(x : IN OUT Big_Signed) IS
  Temp_x : Big_Unsigned := Signed2Unsigned(x);
BEGIN
  IF (x.Last_Significant_Digit = First_Indice) AND THEN
     (x.Mantissa(First_Indice) = 0) THEN
     -- If x = 0...
    x := Big_Signed_One;
    x.Positive := FALSE;
  ELSIF (x.Positive = TRUE) THEN
    -- x > 0
    Dec(Temp_x);
    x := Unsigned2Signed(Temp_x, x.Positive);
  ELSE
    -- x < 0
    Inc(Temp_x);
    x := Unsigned2Signed(Temp_x, x.Positive);
  END IF;
END Dec;

--******************************************************************************

FUNCTION "+"(x, y : IN Big_Signed) RETURN Big_Signed IS
  Tmp_x : Big_Signed := x;
  Tmp_y : Big_Signed := y;
BEGIN
  IF (Tmp_x.Positive /= Tmp_y.Positive) THEN
    IF (Tmp_x.Positive = FALSE) THEN
      Swap1(Tmp_x, Tmp_y);
    END IF;
    Tmp_y.Positive := TRUE;
    RETURN Tmp_x - Tmp_y;
  END IF;

  -- Now, the function studies the case where x and y have the same sign.
  DECLARE
    Temp_x : CONSTANT Big_Unsigned := Signed2Unsigned(Tmp_x);
    Temp_y : CONSTANT Big_Unsigned := Signed2Unsigned(Tmp_y);
    Result : CONSTANT Big_Signed
                            := Unsigned2Signed(Temp_x + Temp_y, Tmp_x.Positive);
  BEGIN
    RETURN Result;
  END;
END "+";

--******************************************************************************
--******************************************************************************

FUNCTION "-"(x, y : IN Big_Signed) RETURN Big_Signed IS
BEGIN
  IF (x.Positive /= y.Positive) THEN
    RETURN x + Opposite(y);
  ELSE              -- (x.Positive = y.Positive)
    DECLARE
      Temp_x : CONSTANT Big_Unsigned := Signed2Unsigned(x);
      Temp_y : CONSTANT Big_Unsigned := Signed2Unsigned(y);
      Result : Big_Signed;
    BEGIN
      IF (Absolute(x) >= Absolute(y)) THEN
        Result := Unsigned2Signed(Temp_x - Temp_y, x.Positive);
      ELSE
        Result := Unsigned2Signed(Temp_y - Temp_x, NOT x.Positive);
      END IF;
      RETURN Result;
    END;
  END IF;
END "-";

--******************************************************************************
--******************************************************************************

FUNCTION "*"(x, y : IN Big_Signed) RETURN Big_Signed IS
  Temp_x : CONSTANT Big_Unsigned := Signed2Unsigned(x);
  Temp_y : CONSTANT Big_Unsigned := Signed2Unsigned(y);
  Result : CONSTANT Big_Signed
             := Unsigned2Signed(Temp_x * Temp_y, (x.Positive = y.Positive));
BEGIN
  RETURN Result;
END "*";

--******************************************************************************
--******************************************************************************

FUNCTION "**"(x : IN Big_Signed;
              y : IN My_Type) RETURN Big_Signed IS
  Temp_Result : Big_Signed := x;
BEGIN
  IF (y = 0) THEN
    RETURN Big_Signed_One;
  END IF;
  IF (x.Last_Significant_Digit = First_Indice)
     AND THEN (x.Mantissa(First_Indice) <= 1) THEN
    IF (Temp_Result.Positive = FALSE) THEN
      -- In this case, the result is positive if y MOD 2 = 0,
      -- and negative if y MOD 2 = 0.
      Temp_Result.Positive := (y MOD 2) = 0;
    END IF;
    RETURN Temp_Result;
  END IF;
  FOR I IN 2..y LOOP
    Temp_Result := Temp_Result * x;
  END LOOP;
  RETURN Temp_Result;
END "**";

--******************************************************************************
--******************************************************************************

FUNCTION "**"(x : IN Big_Signed;
              y : IN Index_Type) RETURN Big_Signed IS
  Temp_Result : Big_Signed := x;
BEGIN
  IF (y = 0) THEN
    RETURN Big_Signed_One;
  END IF;
  IF (x.Last_Significant_Digit = First_Indice)
     AND THEN (x.Mantissa(First_Indice) <= 1) THEN
    IF (Temp_Result.Positive = FALSE) THEN
      -- In this case, the result is positive if y MOD 2 = 0,
      -- and negative if y MOD 2 = 0.
      Temp_Result.Positive := (y MOD 2) = 0;
    END IF;
    RETURN Temp_Result;
  END IF;
  FOR I IN 2..y LOOP
    Temp_Result := Temp_Result * x;
  END LOOP;
  RETURN Temp_Result;
END "**";

--******************************************************************************
--******************************************************************************

FUNCTION "**"(x : IN Big_Signed;
              y : IN Big_Unsigned) RETURN Big_Signed IS
  Temp_Result : Big_Signed := x;
  Indice : Big_Unsigned := Big_Unsigned_One;
BEGIN
  IF (y.Last_Significant_Digit = First_Indice) AND THEN
     (y.Mantissa(First_Indice) = 0) THEN
     -- These two tests are equivalent to the
     -- test "y = Big_Unsigned_Zero", but it's much faster that way.
    RETURN Big_Signed_One;
  END IF;
  IF (x.Last_Significant_Digit = First_Indice) AND THEN
     (x.Mantissa(First_Indice) <= 1) THEN
    IF (Temp_Result.Positive = FALSE) THEN
      -- In this case, the result is positive if y MOD 2 = 0,
      -- and negative if y MOD 2 = 0.
      -- We have just to check if the lowest digit is a multiple or not of 2.
      Temp_Result.Positive := (y.Mantissa(First_Indice) MOD 2) = 0;
    END IF;
    RETURN Temp_Result;
  END IF;
  WHILE (Indice < y) LOOP
    Inc(Indice);
    Temp_Result := Temp_Result * x;
  END LOOP;
  RETURN Temp_Result;
END "**";

--******************************************************************************
--******************************************************************************

FUNCTION "**"(x, y : IN Big_Signed) RETURN Big_Signed IS
  Temp_Result : Big_Signed := x;
  Indice : Big_Signed := Big_Signed_One;
  Temp_y : CONSTANT Big_Unsigned := Signed2Unsigned(y);
BEGIN
  IF (y.Positive = FALSE) THEN
    RETURN Big_Signed_Zero;
  END IF;
  RETURN (x ** Temp_y);
END "**";

--******************************************************************************
--******************************************************************************

PROCEDURE Big_Div(x, y : IN Big_Signed;
                  Quot, Remainder : OUT Big_Signed) IS
  Temp_x : CONSTANT Big_Unsigned := Signed2Unsigned(x);
  Temp_y : CONSTANT Big_Unsigned := Signed2Unsigned(y);
  Temp_Quot, Temp_Remainder : Big_Unsigned;
  Result_Positive : CONSTANT BOOLEAN := (x.Positive = y.Positive);
BEGIN
  Big_Div(Temp_x, Temp_y, Temp_Quot, Temp_Remainder);
  Quot := Unsigned2Signed(Temp_Quot, Result_Positive);
  Remainder := Unsigned2Signed(Temp_Remainder, Result_Positive);
END Big_Div;

--******************************************************************************
--******************************************************************************

PROCEDURE Short_Div(x : IN Big_Signed; y : IN My_Type;
                    Quot : OUT Big_Signed; Remainder : OUT My_Type) IS
  Temp_x : CONSTANT Big_Unsigned := Signed2Unsigned(x);
  Temp_Quot : Big_Unsigned;
BEGIN
  Short_Div(Temp_x, y, Temp_Quot, Remainder);
  Quot := Unsigned2Signed(Temp_Quot, x.Positive);
END Short_Div;

--******************************************************************************
--******************************************************************************

FUNCTION "/"(x, y : IN Big_Signed) RETURN Big_Signed IS
  Quot, Remainder : Big_Signed;
BEGIN
  Big_Div(x, y, Quot, Remainder);
  RETURN Quot;
END "/";

--******************************************************************************
--******************************************************************************

FUNCTION "MOD"(x, y : IN Big_Signed) RETURN Big_Signed IS
  Quot, Remainder : Big_Signed;
BEGIN
  Big_Div(x, y, Quot, Remainder);
  IF (x > Big_Signed_Zero) THEN
    RETURN Absolute(Remainder);
  ELSE
    RETURN Absolute(Absolute(y) - Absolute(Remainder));
  END IF;
END "MOD";

--******************************************************************************
--******************************************************************************

FUNCTION "/"(x : IN Big_Signed; y : IN My_Type) RETURN Big_Signed IS
  Quot : Big_Signed;
  Remainder : My_Type;
BEGIN
  Short_Div(x, y, Quot, Remainder);
  RETURN Quot;
END "/";

--******************************************************************************
--******************************************************************************

FUNCTION "MOD"(x : IN Big_Signed; y : IN My_Type) RETURN My_Type IS
  Quot : Big_Signed;
  Remainder : My_Type;
BEGIN
  Short_Div(x, y, Quot, Remainder);
  IF (x > Big_Signed_Zero) THEN
    RETURN Remainder;
  ELSE
    RETURN (y - Remainder);
  END IF;
  RETURN Remainder;
END "MOD";

--******************************************************************************
--******************************************************************************

FUNCTION Opposite(x : IN Big_Signed) RETURN Big_Signed IS
  Temp_Result : Big_Signed := x;
BEGIN
  IF (Temp_Result.Last_Significant_Digit /= First_Indice) OR ELSE
     (Temp_Result.Mantissa(First_Indice) /= 0) THEN
    -- If Temp_Result /= 0, then inverse the sign.
    -- Note that if x = 0, by convention, the sign of x is always
    -- set to positive.
    Temp_Result.Positive := NOT Temp_Result.Positive;
  END IF;
  RETURN Temp_Result;
END Opposite;

--******************************************************************************
--******************************************************************************

FUNCTION Absolute(x : IN Big_Signed) RETURN Big_Signed IS
BEGIN
  IF (x.Positive = FALSE) THEN
    RETURN Opposite(x);
  ELSE
    RETURN x;
  END IF;
END Absolute;

--******************************************************************************
--******************************************************************************
--**                           Special functions                              **
--******************************************************************************
--******************************************************************************

FUNCTION Number_of_Bytes(x : IN Big_Signed) RETURN Largest_Unsigned IS
BEGIN
  RETURN Number_of_Bytes(Signed2Unsigned(x));
END Number_of_Bytes;

--******************************************************************************

FUNCTION Number_of_Bits(x : IN Big_Signed) RETURN Largest_Unsigned IS
BEGIN
  RETURN Number_of_Bits(Signed2Unsigned(x));
END Number_of_Bits;

--******************************************************************************

FUNCTION Square_Root(x : IN Big_Signed) RETURN Big_Signed IS
  Result, Temp_Result : Big_Signed;
  Two : CONSTANT Big_Signed := (Positive => TRUE,
                                Last_Significant_Digit => First_Indice,
                                Mantissa => (2, OTHERS => 0));
  -- Remark :
  --   In order to avoid the use of a fifth variable, Diff is initialized to 3.
  Diff : Big_Signed := (Positive => TRUE,
                        Last_Significant_Digit => First_Indice,
                        Mantissa => (3, OTHERS => 0));
BEGIN
  IF (x < Big_Signed_Zero) THEN
    RAISE Parameter_Error;
  ELSIF (x.Last_Significant_Digit = First_Indice) AND THEN
        (x.Mantissa(First_Indice) = 0) THEN
    RETURN Big_Signed_Zero;
  ELSIF (x <= Diff) THEN  -- On a provisoirement : Diff = 3.
    RETURN Big_Signed_One;
  END IF;

  -- Loop to calculate the square root
  Result := x / Two;
  WHILE (Diff > Big_Signed_One) LOOP
    Temp_Result := Result;
    Result := (Temp_Result + x/Temp_Result) / Two;
    IF (Temp_Result > Result) THEN
      Diff := Temp_Result - Result;
    ELSE
      Diff := Result - Temp_Result;
    END IF;
  END LOOP;
  RETURN Result;
END Square_Root;

--******************************************************************************

FUNCTION Factorial(x : IN Big_Signed) RETURN Big_Signed IS
  Temp_x : CONSTANT Big_Unsigned := Signed2Unsigned(x);
BEGIN
  IF (x.Positive = FALSE) THEN
    RAISE Parameter_Error;
  END IF;
  RETURN Unsigned2Signed(Factorial(Temp_x), TRUE);
END Factorial;

--******************************************************************************

FUNCTION GCF(x, y : IN Big_Signed) RETURN Big_Signed IS
  -- In english : GCF = greatest common factor
  Temp_x : CONSTANT Big_Unsigned := Signed2Unsigned(x);
  Temp_y : CONSTANT Big_Unsigned := Signed2Unsigned(y);
BEGIN
  RETURN Unsigned2Signed(GCF(Temp_x, Temp_y), TRUE);
END GCF;

--******************************************************************************

FUNCTION LCM(x, y : IN Big_Signed) RETURN Big_Signed IS
  -- finds the Least Common Multiple between 2 arguments : LCM
  Tmp : Big_Signed := (x / GCF(x,y)) * y;
BEGIN
  IF (Tmp.Positive = FALSE) THEN
    Tmp.Positive := TRUE;
  END IF;
  RETURN Tmp;
END LCM;

--******************************************************************************

END Signed_Number;
